/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2009 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person 
 *  obtaining a copy of this software and associated documentation 
 *  files (the "Software"), to deal in the Software without 
 *  restriction, including without limitation the rights to use, copy, 
 *  modify, merge, publish, distribute, sublicense, and/or sell copies 
 *  of the Software, and to permit persons to whom the Software is 
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be 
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind, 
 *  express or implied, including but not limited to the warranties of 
 *  merchantability, fitness for a particular purpose and 
 *  noninfringement. In no event shall the authors or copyright 
 *  holders be liable for any claim, damages or other liability, 
 *  whether in an action of contract, tort or otherwise, arising from, 
 *  out of or in connection with the software or the use or other 
 *  dealings in the software.
 */

#ifndef CROW_DOCUMENT_H
#define CROW_DOCUMENT_H

#include "util.h"

namespace Crow {

const int GuiNeedsUpgrade = 9;

class Document : public Parser {

public:
	Document();

	const Glib::ustring * getError() { return &error; }

	typedef sigc::slot<NodeTest, PNode> NodeSave;

	Glib::ustring save(Model * model, const Nodes & nodes, const NodeSave & nodeSave);
	bool load(Model * model, const Glib::ustring & data, Nodes & loaded);

protected:
	class Upgrade : public RuntimeError {
	public:
		Upgrade() : RuntimeError("upgrade") { }
	};

	Glib::ustring save(PNode node, int indent);

	StateFlags getNodeState(const std::string & name);
	void onStartElement(const Glib::ustring & name, const AttributeMap & properties);
	void onEndElement(const Glib::ustring & name);
	void onText(const Glib::ustring & text);
	void onPassthrough(const Glib::ustring & text);
	 
	void checkVersion(const std::string & version);

private:
	struct idorder { bool operator () (PNode n1, PNode n2); };

	typedef std::set<PNode, idorder> NodesSorted;

	Model * model;

	NodeSave nodeSave;
	NodeSet nodeSet;

	struct SEntry {
		PNode node;
		Glib::ustring text;
		int leafs;
		SEntry() { leafs= 0; }
		SEntry(const PNode & n, const Glib::ustring & t) { node= n; text= t; leafs= 0; } 
	};

	typedef std::list<SEntry> Stack;
	typedef std::map<std::string, std::string> Names;
	typedef std::list<std::pair<PNode, std::string> > Links;

	Stack stack;
	Names names;
	Links links;
	Nodes objects;

	Glib::ustring error;

};

class GuiUpgrade : public Parser {

public:
	Glib::ustring upgrade(const Glib::ustring & data);

protected:
	struct Node {
		std::string element;
		std::string name;
		std::string type;
		Glib::ustring meta;
		std::list<std::string> flags;
		Glib::ustring text;
		Glib::ustring cdata;
		Node * owner;
		std::list<Node *> domain;
		Node() { owner= NULL; }
	};

	void onStartElement(const Glib::ustring & name, const AttributeMap & properties);
	void onEndElement(const Glib::ustring & name);
	void onText(const Glib::ustring & text);
	void onPassthrough(const Glib::ustring & text);

	Glib::ustring write();
	Glib::ustring write(Node * node, int indent);

	void clean();
	void clean(Node * node);
	Node * find(Node * node, const std::string & name);
	void remove(Node * node);
	void reown(Node * node, Node * owner);

	void upgrade1();
	void upgrade2();
	void upgrade4();
	std::string replaceColons(const std::string & str, const std::string & rep=std::string());
	void upgrade5();
	void upgrade6();
	void upgrade8();
	void upgrade9();

private:
	int version;
	std::list<Node *> stack;
	std::list<Node> model;

};

}

#endif
