/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2009 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef CROW_FORWARD_H
#define CROW_FORWARD_H

#define CROW_VERSION_GUIXML (10)

#define CROW_LIBRARY

#ifdef G_LOG_DOMAIN
	#undef G_LOG_DOMAIN
#endif
#define G_LOG_DOMAIN "Crow"

namespace Crow {

const char * const CrowProjectHome= "http://crow-designer.sourceforge.net";

using namespace std::rel_ops;

typedef guint32 Bits;

typedef int TypeId;
const TypeId InvalidTypeId = 0;

typedef int EditorId;
const EditorId InvalidEditorId = 0;

typedef Bits StateFlags;

const StateFlags sfNone          =         0;
const StateFlags sfWeakLink      =   1 <<  0;
const StateFlags sfReadOnly      =   1 <<  1;
const StateFlags sfHidden        =   1 <<  2;
const StateFlags sfNoEditing     =   1 <<  3;
const StateFlags sfLinkOnly      =   1 <<  4;
const StateFlags sfNoUndo        =   1 <<  5;
const StateFlags sfNoSave        =   1 <<  6;
const StateFlags sfOverwrite     =   1 <<  7;
const StateFlags sfAutoDefault   =   1 <<  8;
const StateFlags sfTranslatable  =   1 <<  9;
const StateFlags sfXml           =   1 << 10;
const StateFlags sfChildDef      =   1 << 11;
const StateFlags sfAllowTM       =   1 << 12;

const StateFlags sfNodeStateMask = sfWeakLink | sfNoUndo;

enum TooltipMode { TOOLTIP_TEXT, TOOLTIP_TEXT_OFF, TOOLTIP_MARKUP, TOOLTIP_MARKUP_OFF };

// -----------------------------------------------------------------------------

// canvasuid

struct UIElement;
class UIDefinitionCanvasEditorWidget;

typedef Glib::RefPtr<UIDefinitionCanvasEditorWidget> PUIDefinitionCanvasEditorWidget;
typedef Glib::RefPtr<UIElement> PUIElement;

// canvaswid

class WidgetCanvasEditorWidget;
class WidgetCanvasEditor;

typedef Glib::RefPtr<WidgetCanvasEditorWidget> PWidgetCanvasEditorWidget;
typedef Glib::RefPtr<WidgetCanvasEditor> PWidgetCanvasEditor;

// containers

class Container;

// controller

class Controller;

// crow_impl

class Designer;
typedef std::map<std::string, Glib::ustring> Settings;

// editors

class CanvasEditor;
class Editor;

typedef Glib::RefPtr<CanvasEditor> PCanvasEditor;
typedef Glib::RefPtr<Editor> PEditor;

void RegisterEditors();

// model

class Node;
class Model;
struct Operation;
struct OperationName;
struct OperationMeta;
struct OperationValue;
struct OperationLink;
struct OperationNode;

typedef Glib::RefPtr<Node> PNode;
typedef std::list<PNode> Nodes;
typedef std::set<PNode> NodeSet;
typedef Glib::RefPtr<Operation> POperation;
typedef Glib::RefPtr<OperationName> POperationName;
typedef Glib::RefPtr<OperationMeta> POperationMeta;
typedef Glib::RefPtr<OperationValue> POperationValue;
typedef Glib::RefPtr<OperationLink> POperationLink;
typedef Glib::RefPtr<OperationNode> POperationNode;

// palette

class PaletteEntry;
class EntityEntry;
class Palette;

typedef std::list<std::pair<GType, std::string> > SignalsQuery;

// polytree

class Polycell;
class Polytree;
class PolycellProxy;
class PolycellText;
class PolycellWidget;
class Polyelem;

typedef Glib::RefPtr<PolycellProxy> PPolycellProxy;
typedef Glib::RefPtr<Polycell> PPolycell;
typedef Gtk::TreeModelColumn<PPolycellProxy> PolycellColumn;
typedef Glib::RefPtr<PolycellText> PPolycellText;
typedef Glib::RefPtr<Polyelem> PPolyelem;

// util

class Id;
class Object;
struct Rectangle;
struct Point;

typedef Glib::RefPtr<Object> PObject;
typedef std::vector<Id> IdPath;
typedef std::list<std::string> StringList;

// session

class Session;
class SessionManager;

typedef Glib::RefPtr<Session> PSession;
typedef std::map<IdPath, PSession> Sessions;

// value

class CAny;

typedef Glib::RefPtr<CAny> Any;
typedef std::vector<Any> AnyVector;

// views

class Property;
class EntityView;
class GlibObjectView;
class GtkWidgetView;
class GtkContainerView;

typedef std::pair<bool, Property *> NodeTest;

typedef Glib::RefPtr<EntityView> PEntityView;
typedef Glib::RefPtr<GlibObjectView> PGlibObjectView;
typedef Glib::RefPtr<GtkWidgetView> PGtkWidgetView;
typedef Glib::RefPtr<GtkContainerView> PGtkContainerView;

void RegisterViews1();
void RegisterViews2();
void RegisterViews3();

// widgets

class EditorWidget;
class PropertyEditorWidget;
class HierarchyEditorWidget;
class DesignDialog;

typedef Glib::RefPtr<EditorWidget> PEditorWidget;
typedef Glib::RefPtr<PropertyEditorWidget> PPropertyEditorWidget;

// -----------------------------------------------------------------------------

typedef Glib::RefPtr<Glib::Object> PGlibObject;
typedef Glib::RefPtr<Pango::Layout> PPangoLayout;
typedef Glib::RefPtr<Gdk::Pixmap> PGdkPixmap;
typedef Glib::RefPtr<Gdk::Window> PGdkWindow;
typedef Glib::RefPtr<Gdk::Drawable> PGdkDrawable;
typedef Glib::RefPtr<Gdk::GC> PGdkGC;
typedef Glib::RefPtr<Gtk::TreeModel> PGtkTreeModel;
typedef Glib::RefPtr<Gtk::TreeStore> PGtkTreeStore;
typedef Glib::RefPtr<Gtk::ListStore> PGtkListStore;
typedef Glib::RefPtr<Gtk::Widget> PGtkWidget;
typedef Glib::RefPtr<Gtk::UIManager> PGtkUIManager;
typedef Glib::RefPtr<Gtk::ActionGroup> PGtkActionGroup;
typedef Glib::RefPtr<Gtk::Bin> PGtkBin;
typedef Glib::RefPtr<Gtk::Container> PGtkContainer;
typedef Glib::RefPtr<Gtk::AccelGroup> PGtkAccelGroup;
typedef Glib::RefPtr<Gtk::Action> PGtkAction;
typedef Glib::RefPtr<Gtk::Window> PGtkWindow; //???
typedef Glib::RefPtr<Gdk::Pixbuf> PGdkPixbuf;
typedef Glib::RefPtr<Gtk::Style> PGtkStyle;

}

#endif
