/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2009 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#include "headers.h"
#include "container.h"

void OnNewConnection(
	DBusServer * server,
	DBusConnection * connection,
	void * data
) {

	static gboolean connected= FALSE;

	if(!connected) {

		DBusGConnection * bus= NULL;
		CrowContainer * obj= NULL;

		connected= TRUE;

	  dbus_connection_ref(connection);
	  dbus_connection_setup_with_g_main(connection, NULL);

	  bus= dbus_connection_get_g_connection(connection);

	  obj= ContainerGet();
	  dbus_g_connection_register_g_object(bus, "/CrowContainer", G_OBJECT(obj));

	} else {

		dbus_connection_close(connection);

	}

}

int main(int argc, char ** argv) {

	DBusServer * server;
	DBusError error;
	char * address= NULL;
	GOptionContext * context;
	GOptionEntry entries[]= {
		{ "address", 'a', 0, G_OPTION_ARG_STRING, &address, "DBus address", "A" },
		{ NULL } };

	gtk_init(&argc, &argv);

	context= g_option_context_new("- Crow Designer DBus Service");
	g_option_context_add_main_entries(context, entries, NULL);
	g_option_context_parse(context, &argc, &argv, NULL);

	if(!address) {
		g_printerr("Missing DBus address\n");
		g_option_context_free(context);
		exit(EXIT_FAILURE);
	}

	crow_enter_check("main()");
	ContainerEnter();

	dbus_error_init(&error);
	server= dbus_server_listen(address, &error);
	if(!server) {
		if(dbus_error_has_name(&error, DBUS_ERROR_ADDRESS_IN_USE))
			g_error("DBUS_ERROR_ADDRESS_IN_USE: %s\n", error.message);
		else
			g_error("Creating server failed: %s\n", error.message);
	}

	dbus_server_set_new_connection_function(server, OnNewConnection, NULL, NULL);
	dbus_server_setup_with_g_main(server, NULL);

	g_print("%s\n", address);
	gtk_main();

	dbus_server_disconnect(server);
	dbus_server_unref(server);
	ContainerLeave();
	crow_leave();

	g_option_context_free(context);

	return 0;
}
