/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2009 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#include "headers.h"
#include "util.h"

void CheckFailed(const char * expr, const char * file, int line) {
	char msg[256];
	g_snprintf(msg, sizeof(msg), "Check failed: %s, file: %s, line: %d", expr, file, line);
	FatalError(msg);
}

void FatalError(const char * message) {
	GtkWidget * dialog;
	g_critical(message);

	#ifdef CROW_SANDBOX
		assert(0);
	#endif

	dialog= gtk_message_dialog_new(
		NULL,
		GTK_DIALOG_MODAL,
		GTK_MESSAGE_ERROR,
		GTK_BUTTONS_CLOSE,
		"Fatal Error [" G_LOG_DOMAIN "]"
	);

	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(dialog), message);
	gtk_window_set_keep_above(GTK_WINDOW(dialog), TRUE);
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	abort();
}

int NoOp(void) {
	return 0;
}
